/*----------------------------------------------------------
 * ui-showRecord.js
 * author Ryuto Kurokawa
 * 
 *----------------------------------------------------------*/
$.widget("ui.showRecord",{
	options: {
		attrPrefix: "ui-showRecord-",
		listClassName: "list",
		detailsId: "",
		mapsId: "",
		detailsDateId: "date",
		detailsTimeId: "time",
		detailsMilleageId: "milleage",
		recordSaveId: "saveRecord",
		recordDeleteId: "deleteRecord",
		unsavedData:"",
		currentDataIndex:"",
		dataKeyArray:[]
	},
	_create:function(){
		this.widgetEventPrefix = "record.";
		return this;
	},
	_init:function(){
		var o = this.options;
		if(o.detailsId == "" || o.mapsId == "") return this;

		//set Prefix
		o.listClassName = o.attrPrefix + o.listClassName;
		o.detailsDateId = o.attrPrefix + o.detailsDateId;
		o.detailsTimeId = o.attrPrefix + o.detailsTimeId;
		o.detailsMilleageId = o.attrPrefix + o.detailsMilleageId;

		//レコード詳細表示画面のDOMの生成
		var $dl = $('<dl class="clearfix"> <dt>Date :</dt> <dd id="' + o.detailsDateId + '"></dd> <dt>Time :</dt> <dd id="' + o.detailsTimeId + '"></dd> <dt>Milleage :</dt> <dd id="' + o.detailsMilleageId + '"></dd> </dl> <div id="' + o.recordSaveId + '">SAVE</div> <div id="' + o.recordDeleteId + '">DELETE</div>');


		//生成したDOMの変数保持
		this.$details = $("#" + o.detailsId);
		this.$maps = $("#" + o.mapsId);

		this.$details.append($dl);

		this.$detailsDate = $("#" + o.detailsDateId);
		this.$detailsTime = $("#" + o.detailsTimeId);
		this.$detailsMilleage = $("#" + o.detailsMilleageId);
		this.$recordSave = $("#" + o.recordSaveId);
		this.$recordDelete = $("#" + o.recordDeleteId);

		if(!localStorage) {
			alert("localStorageが使用できません");
			return this;
		}

		//すでに保存済みのレコードがある場合はキー名を配列で管理
		for(var i in localStorage){
			this.options.dataKeyArray.push(i);
		}
		//キー名は保存時の Date.getTime の値を使用
		//レコードの一覧表示は上から新しい順なので、ソートして日時の新しい順で管理
		this.options.dataKeyArray.sort();
		this.options.dataKeyArray.reverse();
		return this;
	},
	showRecordList: function(){
		var dataKeyArray = this.options.dataKeyArray;
		var l = dataKeyArray.length;

		//リストの初期化
		$(this.element).children().remove();

		//保存データなし
		if(l == 0) {
			var elem = $("<li />").text("まだなにもありません");
			$(this.element).append(elem);
			return this;
		}
		
		//保存データあり
		var _this = this;
		for(var i = 0; i < l; i++){
			var obj = JSON.parse(localStorage[dataKeyArray[i]]);
			//走行日時
			var recordDate =	obj.date.year + "/" + 
								obj.date.month + "/" + 
								obj.date.day + " " + 
								obj.date.hour + ":" + 
								obj.date.minutes + ":" + 
								obj.date.second;
			var elem = $("<li />").text(recordDate)
									.addClass(this.options.listClassName);

			//詳細表示時に選択されたデータのキーを使用してデータの詳細を表示するため
			//クリック時に選択されたデータのキーを設定
			elem.bind("click",{index:i},function(e){
				_this.options.currentDataIndex = e.data.index;
			});

			//生成したレコード一覧のDOMの表示
			$(this.element).append(elem);
		}

		return this;
	},
	showResultDetails: function(data){
		//保存前の、走行結果の表示なのでSAVEボタンを表示
		this.$recordDelete.css("display","none");
		this.$recordSave.css("display","block");

		//走行結果の表示と走行結果のデータの一時的な変数保存
		//これはMap表示にもデータを使用するため
		this._createDetails(data);
		this.options.unsavedData = data;
		this.options.currentDataIndex = "";

		//保存ボタンのイベントハンドラの登録
		this.$recordSave.unbind("click");
		this.$recordSave.bind("click",{record:data},$.proxy(this._saveHandler,this));
		return this;
	},
	showRecordDetails: function(){
		if(this.options.currentDataIndex === "") return this;

		//保存済みの走行結果の表示なのでDELETEボタンを表示
		this.$recordSave.css("display","none");
		this.$recordDelete.css("display","block");

		//リスト一覧で選択されたレコードのインデックスを使って表示対象のデータを取得、表示
		var key = this.options.dataKeyArray[this.options.currentDataIndex];
		var data = JSON.parse(localStorage[key]);
		this._createDetails(data);
		
		//削除ボタンのイベントハンドラの登録
		this.$recordDelete.unbind("click");
		this.$recordDelete.bind("click",{key:key},$.proxy(this._deleteHandler,this));

		return this;
	},
	showRecordMap : function(){

		//保存前データのマップ表示の場合はcurrentDataIndexが空なので走行データの一時保存先のunsavedDataを使う
		if(this.options.currentDataIndex === "") {
			var loc = this.options.unsavedData.locations;
		} else {
			var key = this.options.dataKeyArray[this.options.currentDataIndex];
			var loc = JSON.parse(localStorage[key]).locations;
		}
		
		//geolocatioで最初に取得した走行開始位置の緯度と経度を中心とした、Googleマップの作成
		var latlng = new google.maps.LatLng(loc[0].lat,loc[0].lon);
		var mapObj = new google.maps.Map(this.$maps[0], {
			zoom: 16,
			center: latlng,
			mapTypeId: google.maps.MapTypeId.ROADMAP
		});
		
		//作成したGoogleマップの中心、走行開始位置にマーカーの表示
		var marker = new google.maps.Marker({map: mapObj, position:mapObj.getCenter()});
		
		//走行地点の各位置情報で配列を生成
		var pathPoints = [];
		for(var i = 0; i < loc.length; i++){
			var point = new google.maps.LatLng(loc[i].lat,loc[i].lon);
			pathPoints.push(point);
		}
		
		//生成した走行地点の配列を使い、走行経路をポリラインで表示
		var flightPath = new google.maps.Polyline({
			path: pathPoints,
			strokeColor: "#FF0000",
			strokeOpacity: 1.0,
			strokeWeight: 2
		});
		flightPath.setMap(mapObj);

		return this;
	},
	refresh: function(num){
		//Mapのheightを再設定
		this.$maps.css("height",num + "px");

		return this;
	},
	_createDetails: function(data){
		//走行日時
		var recordDate =	data.date.year + "/" + 
							data.date.month + "/" + 
							data.date.day + " " + 
							data.date.hour + ":" + 
							data.date.minutes + ":" + 
							data.date.second;

		//走行時間
		var recordTime = data.time.hour + ":" + data.time.min + ":" + data.time.sec;

		//走行日時、走行時間、走行距離の表示
		this.$detailsDate.text(recordDate);
		this.$detailsTime.text(recordTime);
		this.$detailsMilleage.text(data.milleage);

	},
	_saveHandler:function(e){
		if (!window.confirm("このレコードを保存しますか？")) return;

		this.$recordSave.unbind("click");

		//保存したいデータセットをJSONに変換
		var json = JSON.stringify(e.data.record,null," ")
		//保存したいデータセットのキー名を設定、管理用の配列の先頭に追加
		//キー名は保存時の getTime を使用することで、ユニークかつ保存データのソートもしやすくなる
		var key = new Date().getTime() + "";
		this.options.dataKeyArray.unshift(key);

		//最大保存件数位以上になった場合に一番古いデータを削除
		if(this.options.dataKeyArray.length > 5){
			var deleteKey = this.options.dataKeyArray[this.options.dataKeyArray.length-1];
			delete localStorage[deleteKey];
			this.options.dataKeyArray.pop();
		}

		//レコードの保存と保存時のカスタムイベントの発火
		localStorage[key] = json;
		this._trigger("saved");
	},
	_deleteHandler:function(e){
		if (!window.confirm("このレコードを削除しますか？")) return;

		this.$recordDelete.unbind("click");

		//レコードの削除と削除時のカスタムイベントの発火
		delete localStorage[e.data.key];
		this.options.dataKeyArray.splice(this.options.currentDataIndex,1);
		this._trigger("deleted");
		
	},
});


