/*----------------------------------------------------------
 * ui-lockView.js
 * author Ryuto Kurokawa
 * 
 *----------------------------------------------------------*/
$.widget("ui.lockView",{
	options: {
		attrPrefix: "ui-lockView-",
		headerId: "header",
		middleId: "middle",
		footerId: "footer",
		buttonId: "bt",
		textId: "txt",
		buttonAnimationClass: "backButton",
		textAnimationClass: "backText",
		lightClassName: "light",
		wrapperId: "wrapper",
		title: "Title",
		buttonText : "slide to unlock",
		buttonImgSrc: "images/lockView_bt.png",
		lockedEvent : "locked",
		openedEvent: "opened",
		lockFlg: false,
		animationFlg: 0,
		clearLights:[],
		wrapperWidth: 0,
		buttonWidth: 0,
		minX: 0,
		maxX: 0,
		diff: 0
	},
	_create:function(){
		this.widgetEventPrefix = "lock.";
		return this;
	},
	_init:function(){
		var o = this.options;
		//各ID名クラス名に接頭辞を追加
		o.headerId = o.attrPrefix + o.headerId;
		o.middleId = o.attrPrefix + o.middleId;
		o.footerId = o.attrPrefix + o.footerId;
		o.buttonId = o.attrPrefix + o.buttonId;
		o.textId = o.attrPrefix + o.textId;
		o.wrapperId = o.attrPrefix + o.wrapperId;
		o.buttonAnimationClass = o.attrPrefix + o.buttonAnimationClass;
		o.textAnimationClass = o.attrPrefix + o.textAnimationClass;

		//以下widgetが適用されたDOMを中心に必要なDOMの生成
		this.$lockview = $(this.element);

		//ロック画面中央の背景のDOMをwidget適用対象のDOMをラップするように生成
		this.$middle = $(this.element).children().wrapAll('<div id="' + o.middleId + '">').parent();

		//headerのDOMを生成、header内に引数で受け取ったタイトルのテキストをセット
		this.$header = $('<div id="' + o.headerId + '">')
							.append("<p>" + o.title + "</p>")
							.prependTo(this.element);

		//footerの生成
		this.$footer = $('<div id="' + o.footerId + '">').appendTo(this.element);

		//ロック解除のロック解除ボタンのラッパーとボタンの生成
		this.$wrapper = $('<div id="' + o.wrapperId + '">');
		this.$button = $('<img />').attr({
							src : o.buttonImgSrc,
							id: o.buttonId
						}).appendTo(this.$wrapper);

		//引数でセットされたfooter内の光るテキストを一文字ずつspanタグで囲ってDOMにセット
		this.$buttonText = $('<div id="' + o.textId + '">').appendTo(this.$wrapper);
		$.each(o.buttonText,$.proxy(function(i,v){
			$("<span />").text(v).appendTo(this.$buttonText);
		},this));

		this.$footer.append(this.$wrapper);

		//画面回転時の処理
		$(window).bind("orientationchange resize",$.proxy(this._orientation,this));

		//各要素のアニメーション終了時の処理
		this.$header.bind("webkitTransitionEnd",$.proxy(this._animationEnd,this));
		this.$middle.bind("webkitTransitionEnd",$.proxy(this._animationEnd,this));
		this.$footer.bind("webkitTransitionEnd",$.proxy(this._animationEnd,this));
		this.$footer.children().bind("webkitTransitionEnd",function(e){ return false; });
		
		//ロック解除ボタンのスライド時のtouchmoveイベントとスクロールのイベントの干渉を防ぐ
		$(document).bind("touchmove",function(e){
			e.preventDefault();
		});

		return this;
	},
	lock:function(){
		//ロック解除ボタンへのイベントハンドラの設定用関数の呼び出し
		this._buttonAddEvent();

		//ロック中フラグ
		this.options.lockFlg = true;

		//header footer ロック画面背景各種のpositionとopacityを変更
		//それぞれCSSのtransitionを使ってアニメーションされる
		this.$lockview.css("top", "0");
		this.$header.css("top","0");
		this.$middle.css("opacity","1");
		this.$footer.css("bottom","0");

		//ロック解除ボタンの横のテキストの色のアニメーションの開始
		this._startLight();

		return this;
	},
	reset: function(){
		//ロック解除ボタンの画面上の位置の再計算
		var pos = this.$wrapper[0].getBoundingClientRect();
		this.options.wrapperWidth = this.$wrapper[0].clientWidth - 2;
		this.options.buttonWidth = this.$button[0].clientWidth;
		
		//ロック解除ボタンのスライド幅の設定
		this.options.minX = pos.left * 1;
		this.options.maxX = pos.right * 1;
		this.$button.css("left","0");
		this.$buttonText.css("opacity","1");
	},
	_buttonAddEvent: function(){
		//ロック解除ボタンに各種イベントハンドラの設定
		this.$button.bind("touchstart",$.proxy(this._touchstart,this));
		this.$button.bind("touchmove",$.proxy(this._touchmove,this));
		this.$button.bind("touchend",$.proxy(this._touchend,this));
		this.$button.bind("touchcancel",$.proxy(this._touchcancel,this));
	},
	_buttonRemoveEvent: function(){
		//ロック解除ボタンのイベントハンドラの削除
		this.$button.unbind();
	},
	_orientation: function(e){
		//画面回転時にreset関数を呼ぶ
		this.reset();
	},
	_touchstart: function(e){
		//ロック解除ボタンのクラス名の削除
		this.$button.removeClass();
		//光るテキストのアニメーション停止
		this.$buttonText.removeClass();
		this._stopLight();
		
		//指のタッチ位置と、ボタンの左端との差分を計算、取得
		var touch = e.originalEvent.touches[0];
		var posX = touch.pageX;
		var btnPos = this.$button[0].getBoundingClientRect();
		this.options.diff = posX - btnPos.left;
		
	},
	_touchmove: function(e){

		var touch = e.originalEvent.touches[0];
		var posX = touch.pageX;
		var btnPos = this.$button[0].getBoundingClientRect();
		
		//ロック解除ボタンの両端の位置とスライド幅の可能範囲の比較
		if(this.options.minX <= btnPos.left && this.options.maxX >= btnPos.right) {
			//ロック解除ボタンの左端を設定
			var num = posX - this.options.minX  - this.options.diff;
			
			//ロック解除ボタンの移動後の位置と可能範囲の比較
			if(0 >= num) {
				//スライド可能範囲の左端を超えた場合は左端ピッタリに
				this.$button.css("left","0");
			} else if(this.options.wrapperWidth <= num + this.options.buttonWidth){
				//スライド可能範囲の右端を超えた場合は右端ピッタリに
				this.$button.css("left",this.options.wrapperWidth - this.options.buttonWidth);
			} else {
				//スライド可能範囲内の場合は指の一から計算したロック解除ボタン位置に
				this.$button.css("left",num);
			}
		}
		
		//ロック解除ボタンのスライド位置によって光るテキストの透明度を変更
		var alpha = Math.floor(100 / (this.options.wrapperWidth / 5) * num);
		if(alpha > 100) alpha = 100;
		this.$buttonText.css("opacity",1 - alpha / 100);
		
	},
	_touchend: function(e){
		//touchend時のロック解除ボタンの位置を取得
		var btnPos = this.$button[0].getBoundingClientRect();

		//ロック解除ボタンのスライド可能範囲の右端到達時
		if(btnPos.right >= this.options.maxX - 5) {
			//ロック解除ボタンのイベントハンドラ削除関数の呼び出し
			this._buttonRemoveEvent();
			//ロック解除ボタンを右端ピッタリに
			this.$button.css("left",this.options.wrapperWidth - this.options.buttonWidth);
			//ロック表示解除関数
			this._open();
			return;
		}
		
		//ロック解除ボタンをスライドしてスライド可能範囲の左端に戻す関数の呼び出し
		this._btSlideBack();
	},
	_touchcancel: function(e){
		//touchcancel時はロック解除ボタンを左端に戻す処理
		this._btSlideBack();
	},
	_btSlideBack: function(){
		//transitionが指定されたクラス名をロック解除ボタンに追加
		this.$button.addClass(this.options.buttonAnimationClass);
		//leftの値を左端にすることでtransitionによってアニメーションして左端に戻る
		this.$button.css("left","0");
		
		//光るテキストのアニメーション用のクラス名と透明度の設定
		this.$buttonText.addClass(this.options.textAnimationClass);
		this.$buttonText.css("opacity","1");
		
		//光るテキストのアニメーションスタート関数
		this._startLight();
	},
	_open:function(){
		this.options.lockFlg = false;

		this.$header.css("top","-94px");
		this.$middle.css("opacity","0");
		this.$footer.css("bottom","-95px");

		return this;
	},
	_animationEnd: function(e){
		//ロック画面のheader footer middle 各要素のtransitionアニメーション終了時間数
		//フラグのインクリメント
		this.options.animationFlg++;

		//hedaer footer middle 全てのアニメーション終了のチェック
		if(this.options.animationFlg !== 3) return this;

		this.options.animationFlg = 0;
		this.reset();

		//ロック中フラグによってロック完了かロック解除かの分岐とそれぞれのカスタムイベントの発火
		if(this.options.lockFlg === true){
			this._trigger(this.options.lockedEvent);
		} else {
			this.$lockview.css("top", "-100%");
			this._trigger(this.options.openedEvent);
		}
	},
	_startLight: function(){
		var className = this.options.lightClassName;
		var _this = this;
		var delay = 0;

		//setTimeoutでずらしながらアニメーションのclassを付与
		this.$buttonText.children().each(function(i,elem){
			_this.options.clearLights[i] = setTimeout(function(){
				$(elem).addClass(className);
			},delay);
			delay += 120;
		});
	},
	_stopLight: function(){
		var className = this.options.lightClassName;
		var _this = this;

		//アニメーションのclassの削除とsetTimeoutのclear
		this.$buttonText.children().each(function(i,elem){
			$(elem).removeClass(className);
			if(_this.options.clearLights[i]) clearTimeout(_this.options.clearLights[i]);
		});
	}
});



