/*----------------------------------------------------------
 * geolocation.js
 * author Ryuto Kurokawa
 * 
 *----------------------------------------------------------*/
var Geolocation = function(){
	var w = window,
		d = document,
		n = navigator,
		l = location,
		sDate,
		eDate,
		clearWatch,
		locations;

	if(!n.geolocation){
		alert("ご利用の環境では位置情報APIが使用できません");
	}
	
	function _errorLocation(){
		n.geolocation.clearWatch(clearWatch);
		alert("位置情報が取得できませんでした");
	}

	function _setLocation(loc){
		var latlong = {
			lat : loc.coords.latitude,
			lon : loc.coords.longitude
		};

		//n秒間 位置情報誤差調整のため初期位置を上書き
		var cTime = new Date().getTime();
		if(cTime - sDate.getTime() < 1000 * 3 || !locations[0]){
			locations[0] = latlong;
			return;
		}
		
		//前回取得時と同じ位置情報の場合
		if(locations[locations.length - 1].lat == latlong.lat) return;
		locations.push(latlong);
	}

	function _totalDistance(loc){
		var total = 0;
		
		if(loc.length === 0 || loc.length === 1) return total;
		
		for(var i = 0; i < loc.length-1; i++){
			var a = new google.maps.LatLng(loc[i].lat,loc[i].lon);
			var b = new google.maps.LatLng(loc[i+1].lat,loc[i+1].lon);
			//2点間の距離を計算、代入
			total += google.maps.geometry.spherical.computeDistanceBetween(a,b);
		}
		return Math.floor(total);
	}

	return {
		start : function(){
			locations = [];
			sDate = new Date();
			clearWatch = n.geolocation.watchPosition(
				_setLocation,
				_errorLocation,
				{enableHighAccuracy:true,maximumAge:1}
			);
				
		},
		stop : function(){
			
			//ストップ
			eDate = new Date();
			n.geolocation.clearWatch(clearWatch);
			
			//走行時間の作成
			var el = eDate.getTime() - sDate.getTime();
			el = Math.floor(el / 1000);
			var h = Math.floor(el / 3600) + "";
			var m = Math.floor((el - h * 3600) / 60) + "";
			var s = el - h * 3600 - m * 60 + "";
			
			h = (h.length ===1)? "0" + h : h;
			m = (m.length ===1)? "0" + m : m;
			s = (s.length ===1)? "0" + s : s;
			
			//走行距離の単位調整
			var milleage = _totalDistance(locations);
			milleage = (milleage >= 1000)? milleage/1000 + "km" : milleage + "m";
				
			//走行日時
			var year	 = sDate.getFullYear() + "";
			var month	 = sDate.getMonth() + 1 + "";
			var day		 = sDate.getDate() + "";
			var hour	 = sDate.getHours() + "";
			var minutes	 = sDate.getMinutes() + "";
			var second	 = sDate.getSeconds() + "";
			
			//桁数を揃える
			month	 = (month.length ===1)? "0" + month : month;
			day		 = (day.length ===1)? "0" + day : day;
			hour	 = (hour.length ===1)? "0" + hour : hour;
			minutes	 = (minutes.length ===1)? "0" + minutes : minutes;
			second	 = (second.length ===1)? "0" + second : second;

			//データの生成
			var data = {
				date : {
					year : year,
					month : month,
					day : day,
					hour : hour,
					minutes : minutes,
					second : second
				},
				time : {
					hour	 : h,
					min		 : m,
					sec		 : s,
				},
				locations: locations,
				milleage : milleage
			}
			return data;
		}
	};
	
}();
