/*----------------------------------------------------------
 * ui-slideView.js
 * author Ryuto Kurokawa
 * 
 *----------------------------------------------------------*/
$.widget("ui.slideView",{
	options: {
		attrPrefix: "ui-slideView-",
		wrapperId: "wrapper",
		contentClassName: "content",
		nextClassName: "next",
		backClassName: "back",
		animateClassName: "animate",
		slideEndEvent: "animate_end",
		currentPosition: 0,
		animationFlg: true
	},
	_create:function(){
		this.widgetEventPrefix = "slide.";
		return this;
	},
	_init:function(){
		var o = this.options;
		//各ID名クラス名に接頭辞を追加
		o.wrapperId = o.attrPrefix + o.wrapperId;
		o.contentClassName = o.attrPrefix + o.contentClassName;
		o.nextClassName = o.attrPrefix + o.nextClassName;
		o.backClassName = o.attrPrefix + o.backClassName;
		o.animateClassName = o.attrPrefix + o.animateClassName;

		//各DOM要素にstyleの設定とDOMの編集
		this.$element = $(this.element)
							.css({
								overflow: "hidden"
							});

		this.$contents = $(this.element).children()
							.addClass(o.contentClassName)
							.css({
								float: "left"
							})
							.wrapAll($("<div />"));

		this.$wrapper = this.$contents.parent()
							.attr("id",o.wrapperId)
							.css({
									overflow: "hidden"
							});

		//各子要素の幅を親要素に合わせる
		this._setWidth();
		
		//各コンテンツ感のスライドのアニメーション終了時のイベントハンドラの設定
		this.$wrapper.bind("webkitTransitionEnd",$.proxy(function(){
			//アニメーション用のクラスの削除
			this.$wrapper.removeClass(this.options.animateClassName);
			this.options.animationFlg = true;

			//スライドのアニメーション終了時にカスタムイベントの発火
			this._trigger(this.options.slideEndEvent,0,[this.options.currentPosition]);

		},this));
		return this;
	},
	_setWidth: function(){
		//横並びのDOMのwidth再計算
		this.viewWidth = $(this.element).width();
		this.$wrapper.css("width",this.viewWidth * this.$contents.length);
		this.$contents.css("width",this.viewWidth);

		return this;
	},
	_moveView: function(){
		//transition用のアニメーションのクラスの追加
		this.$wrapper.addClass(this.options.animateClassName);

		//要素の幅 * 要素のインデックス分の移動
		this.$wrapper.css("-webkit-transform", "translate(-" + this.options.currentPosition * this.viewWidth + "px,0px)");

		return this;
	},
	refresh: function(){
		//スライドさせる要素と各コンテンツの要素の幅の再設定とそれに伴うtranslateの再設定
		this._setWidth();
		this.$wrapper.css("-webkit-transform", "translate(-" + this.options.currentPosition * this.viewWidth + "px,0px)");
		return this;
	},
	nextView: function(){
		if(!this.options.animationFlg) return this;

		this.options.animationFlg = false;
		if(this.$contents.length-1 <= this.options.currentPosition) return this;
		this.options.currentPosition++;
		this._moveView();
		return this;
	},
	backView: function(){
		if(!this.options.animationFlg) return this;

		this.options.animationFlg = false;
		if(0 >= this.options.currentPosition) return this;
		this.options.currentPosition--;
		this._moveView();
		return this;
	},
	jumpView: function(num){
		//引数のチェック
		if(this.$contents.length-1 < num || 0 > num) return this;

		//引数の値のインデックスのコンテンツを表示
		this.options.currentPosition = num;
		this.$wrapper.css("-webkit-transform", "translate(-" + this.options.currentPosition * this.viewWidth + "px,0px)");
		return this;
	}
});


