/**
 * JQROID
 * @author daisuke shimizu [ds.ss.ss.fm@gmail.com]
 * @license MIT license
*/
(function($) {

	var win = window;
	var doc = document;

	$(document).bind('mobileinit', function() {
		$.extend( $.mobile, {
		//	ns: 'jqm-',
			loadingMessage: 'ロード中...',
			pageLoadErrorMessage: 'エラーが発生しました'
			
		});
	});

	/**
	 * すべてのページで共通の処理を記述
	 */
	$(".ui-page").live("pagecreate", function(e) {
		$(":jqmData(role='content')", e.currentTarget).height(window.innerHeight);
	});

	var cancelProvisionalHeight = function() {
		$(":jqmData(role='content')", $.mobile.activePage).css("height","");
		$.mobile.silentScroll();
	}

	/**
	 * ホームスクリーン
	 */
	// pagecreateで初期化
	$("#home").live("pagecreate", function(e) {
		// swipeleftでホームスクリーンを左方向にslideさせて切り替える	
		$(".appPanel", e.currentTarget).live("swipeleft", function(){
			$.mobile.changePage($("#home2"), "slide", false, false, false);
		});
	});

	/**
	 * ホームスクリーンの右画面
	 */
	// pagecreateで初期化
	$("#home2").live("pagecreate", function(e) {

		$(".appPanel", e.currentTarget).live("swiperight", function(){
			$.mobile.changePage($("#home"), "slide", true, false, false);
		});

	});

	/**
	 * テーマ設定ページ
	 */
	$("#theme").live("pagecreate", function() {

		$("#selectTheme").bind("change", function(e) {
			e.preventDefault();
			var pages = $("div[data-role='page']");
			var opt = e.target.options[e.target.selectedIndex];
			// CSSのパスを切り替える
			$('#themeCss').attr("href", 'style/'+ opt.value);
		});
	});

	/**
	 * ブラウザ
	 */
	// frame野サイズをデバイスのスクリーンサイズに合わせる
	// pagecreateはliveを使う必要がある
	$("#browser").live("pagecreate", function() {

		var frame = $("#browserFrame").width(win.innerWidth).height(win.innerHeight);

		$("#inputURL").bind("submit", function(e) {
			e.preventDefault();
			browserFrame.location.href = $("#browserUrl").val();
		});
	});
	
	/**
	 * 地図
	 */
	$("#maps").live("pagecreate", function() {

		/**
		 * Google Maps APIのsciprtが読み込まれた後に実行される
		 */
		function mapsLoaded() {
		
			var geocoder = new google.maps.Geocoder();

			var map = $("#mapContent").width(win.innerWidth).height(win.innerHeight);
			var map = $("#mapContent").width("100%").height("100%");

			var mapOptions = {
				zoom: 12,
				mapTypeId: google.maps.MapTypeId.ROADMAP,
				streetViewControl: false,
				mapTypeControl: true,
				mapTypeControlOptions: {
        			style: google.maps.MapTypeControlStyle.HORIZONTAL_BAR,
			        position: google.maps.ControlPosition.RIGHT_BOTTOM
			    }
			};
			var map = new google.maps.Map(doc.getElementById("mapContent"), mapOptions);

			map.setCenter(new google.maps.LatLng(35.68083,139.767036));
			
			var currentMarker = null;

			// Geolocation APIで緯度経度の取得成功
			var locationUpdate = function(pos) {
				var currentLatlng = new google.maps.LatLng(pos.coords.latitude, pos.coords.longitude);

				map.setCenter(currentLatlng);

				if (currentMarker) {
					currentMarker.setPosition(currentLatlng);
				}else{
					currentMarker = new google.maps.Marker({
						position: currentLatlng,
						map: map
					});
				}
				
				/**
				 * Google Masp APIの逆ジオコードを使い緯度、経度から住所を取得する
				 */
				var addressText = "不明";
				if (geocoder) {
					geocoder.geocode({'latLng': currentLatlng}, function(results, status) {
						if (status == google.maps.GeocoderStatus.OK) {
							for (var i=0,len=results.length;i<len;i++) {
								// 広範囲(ex.日本)すぎる住所情報は使わない
								if (results[i].types.length>2) {
									addressText = results[i].formatted_address;
									$(".addressText", $.mobile.activePage).text("住所:" + addressText);
									break;
								}
							}
						}
					});
				}
				$(".addressText", $.mobile.activePage).text("住所:" + addressText);
			};

			// Geolocation APIで位置取得失敗
			var locationFail = function(e) {
			}

			// Geolocation APIで現在地の緯度、軽度を取得する
			if (navigator.geolocation) {
				var watchId = navigator.geolocation.watchPosition(locationUpdate, locationFail);
			}	
		}

		// google maps 初期化
		google.load("maps","3", {"other_params":"sensor=true", "callback" : mapsLoaded});
		
	});

	/**
	 * 写真
	 */
	$("#picture").live("pagebeforecreate", function() {
		// ヘッダーの更新ボタンがタップされた時
		$("#picture .refreshBtn").bind("tap", function(e) {
			e.preventDefault();
			updateList();
		});
	}).live("pageshow", function() {
		// 写真ページが表示された時、グリッドのデータが空であれば更新する
		if ($("#picList").children().length === 0) {
			updateList();
		}
	});

	// 写真更新処理
	var updateList = function() {
		/**
		 * Geolocation APIを使って現在地の緯度経度を取得
		 * その緯度経度から、Instagram APIを使って写真データを検索
		*/

		var dataSize = 0;
		var appendCount = 0;
		var picList = $("#picList").html("");

		// 初期化でloadingが作成されてから、つまりpageshow移行でないと表示できない
		$.mobile.pageLoading();
		
		/**
		 * Instagram APIから指定したIDの写真データを取得、リストビューに追加する
		 *
		 * @param {string} id
		 * @return {void}
		 */
		var getPictureFromId = function(id) {
			$.ajax({
				url: "https://api.instagram.com/v1/locations/" + id + "/media/recent/",
				data: {client_id: "71363a5850c24eca8a7dd79c432b83cc" },
				dataType: "jsonp",
				success: function(data, textStatus, jqXHR) {
					var pics = data.data;
					for (var i=0,len=pics.length;i<len;i++) {
						var thumb = $("<img>").attr("src", pics[i].images.thumbnail.url);
						var x = $("#picList .block").length % 2;
						var inner = $("<div>").addClass("ui-bar").addClass("ui-bar-a").css("padding",".2em 3px").append(thumb);
						picList.append($("<div>").addClass("block").addClass("ui-block-" + ((x==0) ? "a" : "b")).append(inner));
						appendCount++;
					}
					if (appendCount > dataSize) {
						$.mobile.pageLoading(true);
						cancelProvisionalHeight();
					}
			}
			});
		}

		// Instagram APIに緯度経度を渡してデータを検索する
		var getLocationData = function(latitude,longitude) {
			$.ajax({
				url: "https://api.instagram.com/v1/locations/search",
				data: { lat: latitude, lng : longitude, client_id: "71363a5850c24eca8a7dd79c432b83cc" },
				dataType: "jsonp",
				success: function(data, textStatus, jqXHR) {
					var locations = data.data;
					for (var i=0,len=20;i<len;i++) {
						var id = locations[i].id;
						getPictureFromId(id);
					}
				}
			});
		}

		// 緯度経度の取得成功
		var locatonUpdate = function(position) {
			getLocationData(position.coords.latitude, position.coords.longitude);
		}

		// 緯度経度の取得失敗
		var locationFail = function() {
			// 緯度経度の取得に失敗した場合は、固定の緯度経度で検索する
			getLocationData(35.68083,139.767036);
		}

		// Geolocation APIで現在地の緯度、軽度を取得する
		if (navigator.geolocation) {
			navigator.geolocation.getCurrentPosition(locatonUpdate, locationFail, {
				maximumAge:Infinity,
				timeout:5000
			});
			
		}
	}

	/**
	 * 音楽 アーティストリスト
	 */
	$("#artist").live("pageshow", function() {
		$(".refreshBtn", $.mobile.activePage).bind("tap", function(e) {
			e.preventDefault();
			updateArtist();
		});
		if ($("#artistList").children().length === 0) {
			updateArtist();
		}
	});

	var updateArtist = function() {
		
		// リスト作成中はローディングを表示
		$.mobile.pageLoading();
		
		var artistList = $("#artistList");
		var rowTmpl = $("#artistRowTmpl");
		var dividerTmpl = $("#dividerTmpl");

		// Yahoo! Music APIからYQLを使ってアーティスト情報を取得する
		$.ajax({
			url: "http://query.yahooapis.com/v1/public/yql",
			data: {
				q:'select * from music.artist.popular limit 10 | sort (field="name")',
				format: 'json',
				diagnostics: 'false'
			},
			dataType: "jsonp",
			error: function(jqXHR, textStatus, errorThrown) {
			},
			success: function(data, textStatus, jqXHR) {

				var lastIndex = "";
				var data = data.query;
				for (var i=0;i<data.count;i++) {
					var artist = data.results.Artist[i];
					var index = artist.name.charAt(0).toUpperCase();
					// アーティスト名の頭文字を索引として、list-dividerを挿入する
					if (lastIndex !== index) {
						dividerTmpl.tmpl({text: index}).appendTo(artistList);
						lastIndex = index;
					}
					artist.href = "track.html?artist=" + encodeURI(artist.name); 
					rowTmpl.tmpl(artist).appendTo(artistList);
				}
				$.mobile.pageLoading(true);
				// JQMのリストビューを適用するためにはrefreshする必要がある
				artistList.listview('refresh');
				$.fixedToolbars.show(true, $.mobile.activePage);
				cancelProvisionalHeight();
				//$($.mobile.activePage).fixHeaderFooter();
				// 現在のVersionではリストビューをダイナミックに変更するとFooterのfixedがうまくいかないのでfixHeaderFooterを明示的に呼ぶ
			}
		});
	};

	/**
	 * 音楽 曲リスト
	 */
	$("#track").live("pageshow", function() {
		$(".refreshBtn", $.mobile.activePage).bind("tap", function(e) {
			e.preventDefault();
			updateTrack();
		});
		if ($("#trackList").children().length === 0) {
			updateTrack();
		}
	});

	var updateTrack = function() {

		// リスト作成中はローディングを表示
		$.mobile.pageLoading();
		
		var trackList = $("#trackList");
		var rowTmpl = $("#trackRowTmpl");

		// Yahoo! Music APIからYQLを使ってアーティスト情報を取得する
		$.ajax({
			url: "http://query.yahooapis.com/v1/public/yql",
			data: {
				q:'select * from music.track.popular limit 10 | sort (field="name")',
				format: 'json',
				diagnostics: 'false'
			},
			dataType: "jsonp",
			error: function(jqXHR, textStatus, errorThrown) {
			},
			success: function(data, textStatus, jqXHR) {
				var lastIndex = "";
				var data = data.query;
				for (var i=0;i<data.count;i++) {
					var track = data.results.Track[i];
					track.thumb = (track.Album.Release.Image) ? track.Album.Release.Image[1].url : "",
					rowTmpl.tmpl(track).appendTo(trackList);
				}
				$.mobile.pageLoading(true);
				
				// JQMのリストビューを適用するためにはrefreshする必要がある
				trackList.listview('refresh');
				cancelProvisionalHeight();
				$.fixedToolbars.show(true, $.mobile.activePage);
				//$($.mobile.activePage).fixHeaderFooter();
				// 現在のVersionではリストビューをダイナミックに変更するとFooterのfixedがうまくいかないのでfixHeaderFooterを明示的に呼ぶ
			}
		});
	};

})(jQuery);




